from .data_generators.patient_data_generator import PatientDataGenerator
from .datasets.patient_dataset import PatientDataset
from .logging_tools import logs_file_setup

__author__ = "Maxence Larose"
__version__ = "0.1.4"
__copyright__ = "Copyright 2022, Maxence Larose"
__credits__ = ["Maxence Larose"]
__license__ = "GPL"
__maintainer__ = "Maxence Larose"
__email__ = "maxence.larose.1@ulaval.ca"
__status__ = "Production"



"""
    @file:              base_patient_data_factory.py
    @Author:            Maxence Larose

    @Creation Date:     01/2022
    @Last modification: 01/2022

    @Description:       This file contains the class BasePatientDataFactory that is used as an abstract class used as a
                        reference for all other patient data factories.
"""

from abc import ABC, abstractmethod
import os
from typing import Dict, List, Optional

from ....data_model import PatientDataModel
from ...image.dicom_reader import DicomReader


class BasePatientDataFactory(ABC):
    """
    An abstract class that is used as a reference for all other patient data factories.
    """

    def __init__(
            self,
            path_to_images_folder: str,
            path_to_segmentations_folder: Optional[str],
            series_descriptions: Optional[Dict[str, List[str]]],
            verbose: bool
    ):
        """
        Constructor of the class BasePatientDataFactory.

        Parameters
        ----------
        path_to_images_folder : str
            Path to the folder containing the patient's image files.
        path_to_segmentations_folder: Optional[str]
            Path to the folder containing the patient's segmentation files.
        series_descriptions : Optional[Dict[str, List[str]]]
            A dictionary that contains the series descriptions of the images that absolutely needs to be extracted from
            the patient's file. Keys are arbitrary names given to the images we want to add and values are lists of
            series descriptions. The images associated with these series descriptions do not need to have a
            corresponding segmentation. In fact, the whole point of adding a way to specify the series descriptions that
            must be added to the dataset is to be able to add images without segmentation.
        verbose : bool
            True to log/print some information else False.

        Attributes
        ----------
        self._images_data : List[ImageDataModel]
            A list of the patient's images data.
        """
        self._path_to_images_folder = path_to_images_folder
        self._path_to_segmentations_folder = path_to_segmentations_folder
        self._series_descriptions = series_descriptions
        self._verbose = verbose

        dicom_reader = DicomReader(path_to_images_folder=self._path_to_images_folder)
        self._images_data = dicom_reader.get_images_data(self._verbose)

    @property
    def patient_name(self) -> str:
        """
        Patient name.

        Returns
        -------
        patient_name : str
            Patient name.
        """
        patient_name = self._images_data[0].dicom_header.PatientName

        return str(patient_name)

    @property
    def _paths_to_segmentations(self) -> List[str]:
        """
        A list of paths to the segmentation files.

        Returns
        -------
        paths_to_segmentations : List[str]
            A list of paths to the segmentation files.
        """
        paths_to_segmentations = []
        for segmentation_filename in os.listdir(self._path_to_segmentations_folder):
            path_to_dicom_folder = os.path.join(self._path_to_segmentations_folder, segmentation_filename)
            paths_to_segmentations.append(path_to_dicom_folder)

        return paths_to_segmentations

    @abstractmethod
    def create_patient_data(self) -> PatientDataModel:
        """
        Creates a tuple containing all the patient's data.

        Returns
        -------
        patient_data: PatientDataModel
            Patient data.
        """
        pass



"""
    @file:              logging_tools.py
    @Author:            Maxence Larose

    @Creation Date:     10/2021
    @Last modification: 01/2022

    @Description:       This file contains the logs_file_setup function, which allows the user to set the root logger
                        level to the specified level.
"""

import logging

import os
import sys
import time
from datetime import date


def logs_file_setup(level=logging.INFO):
    today = date.today()
    timestamp = str(time.time()).replace('.', '')
    logs_dir = f"logs/logs-{today.strftime('%d-%m-%Y')}"
    logs_file = f'{logs_dir}/{timestamp}.log'
    os.makedirs(logs_dir, exist_ok=True)
    logging.basicConfig(filename=logs_file, filemode='w+', level=level)
    handler = logging.StreamHandler(sys.stdout)
    logging.getLogger().addHandler(handler)



"""
    @file:              settings.py
    @Author:            Maxence Larose

    @Creation Date:     10/2021
    @Last modification: 01/2022

    @Description:       Stores custom enumerations of the important file names, folder names and paths within the
                        project.
"""

from os.path import abspath, dirname, join

ROOT = abspath(dirname(__file__))


class FileName:
    SERIES_DESCRIPTIONS_JSON: str = "series_descriptions.json"
    PATIENT_DATASET_HDF5: str = "patient_dataset.h5"


class FolderName:
    DATA_FOLDER: str = "data"
    PATIENTS_FOLDER: str = "patients"
    IMAGES_FOLDER: str = "images"
    SEGMENTATIONS_FOLDER: str = "segmentations"


class PathName:
    PATH_TO_DATA_FOLDER: str = join(ROOT, FolderName.DATA_FOLDER)

    PATH_TO_SERIES_DESCRIPTIONS_JSON: str = join(PATH_TO_DATA_FOLDER, FileName.SERIES_DESCRIPTIONS_JSON)
    PATH_TO_PATIENT_DATASET_HDF5: str = join(PATH_TO_DATA_FOLDER, FileName.PATIENT_DATASET_HDF5)

    PATH_TO_PATIENTS_FOLDER: str = join(PATH_TO_DATA_FOLDER, FolderName.PATIENTS_FOLDER)
